/*
 * Created on 2004-11-30
 *
 */
package edu.umk.mat.merdacz.JGServer;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.Resource;
import org.xmldb.api.base.ResourceIterator;
import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;
import org.xmldb.api.modules.XPathQueryService;
import org.xmldb.api.modules.XUpdateQueryService;

/**
 * Wrappes all db stuff. We are creating one object of
 * this (one db connection) per player connection.
 * This makes all quite easy (doesn't need to be synchronized
 * because user requests are parsed sequently)
 *  
 * @author Marcin Daczkowski
 *  
 */
public class DbWrapper {
    /** 
     * eXist NXD driver is hardcoded - this should be changed in 
     * production version
     */
    private static String driver = "org.exist.xmldb.DatabaseImpl";

    /**
     * eXist Driver for XML:DB API 
     */
    private Class aClass;

    /**
     * Our servers database instance (using driver from aClass)
     */
    private Database db;
    
    /**
     * Root collection in database for this server (JGSdb currently hardcoded)
     */
    private Collection col;
    
    /**
     * Hardcoded. In future it should be placed in configuration 
     */
    private static String gamesCollectionName = "games";
    
    /**
     * JGSdb/games collection. 
     */
    private Collection game;
    
    /**
     * Hardcoded. In future it should be placed in configuration
     */
    private static String roomsCollectionName = "rooms";
    
    /**
     * JGSdb/rooms collection
     */
    private Collection rooms;
    
    /**
     * JGSdb/games/{gameName}/users collection
     */    
    private Collection users;

    /**
     * Address of db which we got form configuration file.
     */
    private String dbAddress;

    /**
     * User used for db connection (from configuration file)
     */
    private String dbUser;

    /**
     * User password used for db connection (from configuration file)
     */
    private String dbPass;
    
    /**
     * Service to pass XPath and XQuery questions to NXD (now eXist)
     */
    private XPathQueryService xpqService;
    
    /**
     * Service to pass XUpdate queries to NXD (now eXist)
     */
    private XUpdateQueryService xuqService;
    
    /**
     * Wrappes connection to NXD. It is instantinated before user choose
     * exact game, hence we cannot set all stuff yet. When 
     * user chooses game (through setGame request) dbSetGame method *must* be called.
     * We currently do not check it, hence using db* method before calling dbSetGame
     * will cause internal error.
     * 
     * @param dbAddress
     * @param dbUser
     * @param dbPass
     * @throws DbWrapperException
     */
    public DbWrapper(String dbAddress, String dbUser, String dbPass)
            throws DbWrapperException {
        this.dbAddress = dbAddress;
        this.dbUser = dbUser;
        this.dbPass = dbPass;

        try {
            aClass = Class.forName(driver);
            db = (Database) aClass.newInstance();

            DatabaseManager.registerDatabase(db);

            // get main collection
            col = DatabaseManager.getCollection(this.dbAddress, this.dbUser,
                    this.dbPass);
            
            // get users collection, game collection to null 
            // beacuse we don't know which game to use yet (same with rooms)

            game = null;
            rooms = null;
            users = col.getChildCollection("users");

            // get needed service in order to avoid geting them 
            // every time when method which uses it is called 
            xpqService = (XPathQueryService) col.getService("XPathQueryService",
                    "1.0");
            xpqService.setProperty("indent", "no");
            
            // load xqueries && xupdates from ?files to hashmap
            

        } catch (ClassNotFoundException e) {
            throw new DbWrapperException(e.getMessage(), e);

        } catch (InstantiationException e) {
            throw new DbWrapperException(e.getMessage(), e);

        } catch (IllegalAccessException e) {
            throw new DbWrapperException(e.getMessage(), e);

        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);

        } catch (Exception e) {
            throw new DbWrapperException(e.getMessage(), e);
        }

    }
    
    /**
     * Must be called before any other method use.
     * 
     * @param gameName indicates sub-collection of JGSdb/games which to use 
     * @throws DbWrapperException
     * @throws XMLDBException
     */
    public void setGame(String gameName) throws DbWrapperException {
        try {            
            game = col.getChildCollection(gamesCollectionName).getChildCollection(gameName);
            rooms = game.getChildCollection(roomsCollectionName);
            
            // get needed service in order to avoid geting them 
            // every time when method which uses it is called           
            xuqService = 
                (XUpdateQueryService) rooms.getService("XUpdateQueryService", "1.0");
            
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);
            
        }
    }
    
    /**
     * It probably doesn't really need to be called after logout, but
     * calling it is quite good idea.
     * 
     */
    public void unsetGame() {
        game = null;
        rooms = null;
        
    }

    /**
     * Checks whether login name and password match
     * 
     * @param name
     * @param password
     * @return true when login/password match false otherwise
     * @throws DbWrapperException
     */
    public boolean dbCheckLogin(String name, String password) throws DbWrapperException {

        ResourceSet result;
        try {
            result = xpqService.query("//user[login=\"" + name + "\""
                    + " and password=\"" + password + "\"]");

            if (result.getSize() != 1) {
                return false;
            }

        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);
            
        }

        return true;

    }

    /**
     * Registers new user in database. 
     * 
     * TODO (minor) this should take additional parameters like name, surname, avatar
     * 
     * @param name
     * @param password
     * @return true when registration went ok false otherwise
     * @throws DbWrapperException
     */
    public boolean dbRegister(String name, String password) throws DbWrapperException {
        String fileName = name + ".xml";

        String content = "<user>" +
        		"<login>" + name + "</login>" +
        		"<password>" + password + "</password>" +
        		"</user>";
        
        try {
            if ( users.getResource(fileName) == null) {
                XMLResource doc = 
                    (XMLResource) users.createResource(fileName, "XMLResource");
                
                doc.setContent(content);
                users.storeResource(doc);
                
                return true;
            } else {
                return false;
            }
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);   
            
        }
        

    }
    
    /**
     * Adds file {roomName}.xml to JGSdb/games/{gameName}/rooms with proper
     * information about created room.
     * <p>
     * Doesn't we double check room existence?
     * 
     * @param roomName
     * @param roomOwner
     * @param roomType
     * @return false if room already exists true otherwise
     * @throws DbWrapperException
     */
    public boolean dbAddRoom(String roomName, String roomOwner, String roomType) throws DbWrapperException {
        String fileName = roomName + ".xml";
                
        String content = "<room name=\"" + roomName + "\" " +
        		"owner=\"" + roomOwner + "\" type=\"" + roomType + "\">" +
        				"<invitations/></room>";
    
        // check whether proper collection has been binded to "rooms"
        if ( rooms == null ) {
            throw new DbWrapperException("game collection hasn't been properly binded");
        }
            
        try {
            if ( rooms.getResource(fileName) == null) {
                XMLResource doc = 
                    (XMLResource) rooms.createResource(fileName, "XMLResource");
                
                doc.setContent(content);
                rooms.storeResource(doc);
                
                return true;
                
            } else {
                return false;
                
            }
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);   
            
        }        

    }
    
    /**
     * We don't check ownership of room here. This must be done outside.
     * We don't return any information about room existence 
     * (this can be handled only by catching DbWrapperException)
     * 
     * @param roomName 
     * @throws DbWrapperException
     */
    public void dbRemoveRoom(String roomName) throws DbWrapperException {
        String fileName = roomName + ".xml";
        
        if (rooms == null) {
            throw new DbWrapperException("game collection hasn't been properly binded");            
        }
        
        try {
            XMLResource xmlr = (XMLResource) rooms.getResource(fileName);
            
            if ( xmlr != null ) {
                rooms.removeResource(xmlr);                
            } else {
                throw new DbWrapperException("resource " + roomName + 
                        " doesn't exist in rooms collection");
            }            
                        
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);
            
        }             
    }

    
    /**
     * Uses XUpdate to append <code>&lt;invite&gt;</code> element under 
     * <code>&lt;invitations&gt;</code> node in proper roomName room.  
     * 
     * @param playerName
     * @param roomName
     * @throws DbWrapperException
     */
    public void dbAddInvite(String playerName, String roomName) throws DbWrapperException {
        // should be stored in file and loaded to some "static class"
        
        // I have probably found bug in exist (or sth which its XUpdate 
        // implementation uses). Find what couses problem and send email to sb
        // problem is when \" used in select od xu:append, ' are ok!
        
        String xupd = 
    		"<xu:modifications version=\"1.0\" xmlns:xu=\"http://www.xmldb.org/xupdate\">\n" +
    		"<xu:append select=\"/room[@name='" + roomName + "']/invitations\">\n"+
    		"	<xu:element name=\"invite\">\n"+
    		"	<xu:attribute name=\"playerName\">" + playerName + "</xu:attribute> \n"+
    		"	</xu:element> \n"+
    		"</xu:append>"+
    		"</xu:modifications>";    
        
        try {
           
            xuqService.update(xupd); // this should always be == 1
            
            
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);
        }

     
        
    }
    
    /** 
     * Gets rooms from db as XML. Used for restoring rooms from db after JGServer
     * reboot.
     * 
     * @return XML which contains rooms from db
     * @throws DbWrapperException
     */
    public String dbGetRooms() throws DbWrapperException {
        String query = "//room";
        
        String xmlout = "<rooms>";
        ResourceSet result;
        try {
            result = xpqService.query(query);
            ResourceIterator i = result.getIterator();

            while (i.hasMoreResources()) {

                Resource r = i.nextResource();
                xmlout = xmlout + (String) r.getContent();
            }
            
            
        } catch (XMLDBException e) {
            throw new DbWrapperException(e.getMessage(), e);
            
        }
        
        return xmlout + "</rooms>";

    }
  
}