/*
 * Created on 2004-11-17
 *
 */

package edu.umk.mat.merdacz.JGServer;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.StringReader;
import java.net.Socket;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import edu.umk.mat.merdacz.JGServer.config.ServerCfg;

/**
 * Main per connection while-loop
 * 
 * @author Marcin Daczkowski
 *  
 */

public class ServerThread extends Thread {

    /**
     * Reference to the Server object from which this instance of ServerThread
     * had been created. Need this because of serverCfg (we need it for checking
     * whether debug flag is set) and for invoking closing connection routine
     */
    private Server server;

    /**
     * The socket for this concrete connection.
     * 
     * TODO (major) do it with secure sockets layer
     */
    private Socket socket;

    /**
     * Shortcut to server.getGames();
     */
    private Games games;

    /**
     * Shortcut to games.getGameByName(...)
     */
    private Game game;

    /**
     * Shortcut to game.getRooms()
     */
    private Rooms rooms;

    /**
     * The user can be only in one room at most and this is this room.
     * Redundance to Player's private room field. We use it just to have
     * shortcut.
     *  
     */
    private Room actualRoom;

    /**
     * Player object is instantinated when login or register request occurs and
     * is succesfully done
     */
    private Player player;

    /**
     * Reference to ServerCfg object which is needed due to checking if Debug
     * flag is set.
     */
    private ServerCfg cfg;

    /**
     * Indicates whether setgame request has been made We needs this because
     * some requests depends on the fact that a game has been chosen
     */
    private boolean depsSetGame;

    /**
     * Indicates whether player has already logged in. We needs this because
     * some requests depends on the fact that player has logged in
     */
    private boolean depsLoggedIn;

    /**
     * Indcates wheter player is in a room
     */
    private boolean depsInRoom;

    /**
     * Indicates whether the room has been removed We need this because of how
     * message parsing is done.
     */
    private boolean removedRoom;

    /**
     * Indicates whether we has just left the actualRoom We need this because of
     * how message parsing is done.
     */
    private boolean leftRoom;

    /**
     * 
     */
    private boolean gameUnsetted;
    
    /**
     * db stuff
     */
    private DbWrapper db;

    /**
     * @param server
     *            Reference to main server thread from which this one has been
     *            created.
     * @param socket
     *            Socket which is used for communication with the player.
     */
    public ServerThread(Server server, Socket socket) {
        this.server = server;

        this.socket = socket;
        this.games = server.getGames();

        this.cfg = server.getCfg();

        // one request is parsed at time ;) don't have to synchronize
        try {
            db = new DbWrapper(cfg.getDbAddress(), cfg.getDbUser(), 
                    cfg.getDbPass());
            
        } catch (DbWrapperException e) {
            System.out.println(e.getMessage());

        }

        start();
    }

    /**
     * Main ServerThread while-loop
     */
    public void run() {

        depsSetGame = false;
        depsLoggedIn = false;
        removedRoom = false;
        leftRoom = false;
        gameUnsetted = false;

        try {
            DataInputStream din = new DataInputStream(socket.getInputStream());

            player = new Player(new DataOutputStream(socket.getOutputStream()));

            // main per user while-loop
            while (true) {
                String msg = din.readUTF();

                if ((actualRoom != null) && (actualRoom.isRemoved())) {
                    player.leaveRoom();
                    depsInRoom = false;
                    actualRoom = null;
                    if (cfg.getDebug()) {
                        System.out.println(player.getLoginName() + " has "
                                + "been automaticly left from room!");
                    }
                }

                if (cfg.getDebug()) {
                    System.out.println("Recived from " + socket + ": " + msg);
                }

                try {
                    String xmlToSend = parseRequest(msg);
                    // wrapped only because used here and above                    
                    parseActions(xmlToSend);

                } catch (JDOMException e) {
                    // This will occur when we meet parse error
                    // (not validation! - beacuse we do not validate :)

                    if (cfg.getDebug()) {
                        System.out.println("Malformed request!");
                    }
                } catch (DbWrapperException e) {
                    System.out.println(e.getMessage());
                    
                }

            }
        } catch (EOFException e) {
            // This is ok - connection has been closed -> finally
        } catch (IOException e) {
            System.out.println(e);
        } finally {
                       
            try {
                // wrapped only because used here and above
                parseActions(logout(""));
                
                
            } catch (JDOMException e) {
                // Shouldn't happen
                e.printStackTrace();
                
            } catch (IOException e) {
                // Shouldn't happen 
                e.printStackTrace();
                
            }

            server.removeConnection(socket);
        }

    }

    /**
     * @param xmlin
     *            XML message from player
     * @return XML's (one per line) such like reply and broadcasts (in room & in
     *         game)
     * @throws JDOMException
     * @throws IOException
     * @throws DbWrapperException
     */
    private String parseRequest(String xmlin) throws JDOMException,
            IOException, DbWrapperException {

        // We use Java DOM implementation (JDOM). Maybe it is not
        // efficient but... it is easy to use (easier than pure SAX, or
        // pull-parsing)
        SAXBuilder sb = new SAXBuilder();
        StringReader reader = new StringReader(xmlin);

        sb.setValidation(false);

        Document doc = sb.build(reader);
        
        String type = doc.getRootElement().getAttributeValue("type");
        String id = doc.getRootElement().getAttributeValue("id");
                
        // malformed request
        if (!doc.getRootElement().getName().equals("request")) {
            return null;
        }

        // if no id request should be ignored
        if (id == null) {
            return null;
        }
        
        // if id is empty the request should be ignored 
        // (we use empty one for special internal purposes)
        if (id.equals("")) {
            return null;
        }

        // malformed request
        if (type == null) {
            return null;	
        }

        // this is string with answer witch will be returned
        String xmlout;

        // find out what type of request it is
        if (type.equals("setGame")) {
            xmlout = setGame(id, doc.getRootElement().getChild("name"));

        } else if (type.equals("login")) {
            xmlout = login(id, doc.getRootElement().getChild("name"), doc
                    .getRootElement().getChild("password"));

        } else if (type.equals("logout")) {
            xmlout = logout(id);

        } else if (type.equals("register")) {
            xmlout = register(id, doc.getRootElement().getChild("name"), doc
                    .getRootElement().getChild("password"), doc
                    .getRootElement().getChild("passwordAgain"), doc
                    .getRootElement().getChild("avatar"));

        } else if (type.equals("getRooms")) {
            xmlout = getRooms(id);

        } else if (type.equals("joinRoom")) {
            xmlout = joinRoom(id, doc.getRootElement().getChild("name"));

        } else if (type.equals("leaveRoom")) {
            xmlout = leaveRoom(id);
            
        } else if (type.equals("grantInvitation")) {
            xmlout = grantInvitation(id, doc.getRootElement().getChild("playerName"),
                    doc.getRootElement().getChild("roomName"));

        } else if (type.equals("getPlayersInRoom")) {
            xmlout = getPlayersInRoom(id, doc.getRootElement().getChild("name"));

        } else if (type.equals("getPlayersInGame")) {
            xmlout = getPlayersInGame(id);

        } else if (type.equals("newRoom")) {
            xmlout = newRoom(id, doc.getRootElement().getChild("name"), doc
                    .getRootElement().getChild("type"));

        } else if (type.equals("removeRoom")) {
            xmlout = removeRoom(id, doc.getRootElement().getChild("name"));

        } else if (type.equals("getStats")) {
            xmlout = getStats(id);
        } else if (type.equals("chatMsg")) {
            if (depsSetGame && depsLoggedIn && depsInRoom) {
                xmlout = broadcastChatMsg(player.getLoginName(), doc
                        .getRootElement().getText());
            } else {
                xmlout = replyFailed(id, "chatMsg", "no room chosen");
            }

        } else if (type.equals("gamelet")) {

            if (depsSetGame && depsLoggedIn && (actualRoom != null)) {
                // invoke specific gamelet parseRequest

                if (doc.getRootElement().getChild("gameRequest") != null) {
                    // we use clone beacuse we have to deattach gameRequest
                    // element from its parent

                    Document gameDoc = new Document((Element) doc
                            .getRootElement().getChild("gameRequest").clone());
                    
                    String gameletOut = 
                        actualRoom.parseRequest(player, gameDoc);
                    
                    // pack gamelet output in common server reply/broadcast or so
                    if (gameletOut != null) {
                        BufferedReader ia = new BufferedReader(
                                new StringReader(gameletOut));

                        String action;
                        
                        xmlout = "";
                        
                        // read xmlout produced by parserequest line by line
                        while ((action = ia.readLine()) != null) {
                           
                            if (action.substring(1, 10).equals("gameReply")) {
                                xmlout = xmlout + reply(id, "gamelet", action) + "\n";
                                
                            } else { // gameBroadcast
                                xmlout = xmlout + broadcastInRoom("gamelet", action) + "\n";
                            }
                        }
                        
                    } else {
                        if (cfg.getDebug()) {
                            System.out.println("Internal plugin error");
                        }
                        
                        xmlout = "";
                        
                    }
                    
                    
                } else {
                    xmlout = replyFailed(id, "gamelet", "gameRequest element missing");
                }

            } else if ((!depsSetGame) || (!depsLoggedIn)) {
                return replyFailed(id, "gamelet", "not logged in");

            } else {
                return replyFailed(id, "gamelet", "no room chosen");

            }

        } else {
            return replyFailed(id, type, "unknown request type");

        }

        return xmlout;

    }
    
    /**
     * Parse xml's produced by parseRequest and take proper actions
     * 
     * @param xmlToSend
     * @throws IOException
     * @throws JDOMException
     */
    private void parseActions(String xmlToSend) throws JDOMException, IOException {
        if (xmlToSend != null) {
            BufferedReader ia = new BufferedReader(
                    new StringReader(xmlToSend));

            String action;
            // read xmlout produced by parserequest line by line
            while ((action = ia.readLine()) != null) {

                if (action.substring(1, 6).equals("reply")) {
                    player.sendMessage(action);
                    if (cfg.getDebug()) {
                        System.out.println("Sended to " + socket
                                + ": " + action);
                    }
                } else if (action.substring(1, 16).equals(
                        "broadcastInRoom")) {
                    if (actualRoom != null) {            
                        // TODO i have decided not to put player here
                        // because we would need ugly hacks in gamelets!                       
                                                
                        actualRoom.broadcast(action, null);
                        if (cfg.getDebug()) {
                            System.out.println("Broadcasted to "
                                    + actualRoom.getName()
                                    + " except  "
                                    + player.getLoginName() + ": "
                                    + action);
                        }
                    }
                } else if (action.substring(1,16).equals(
                        "broadcastInGame")) {
                
                    // agh! player can be null.
                    game.broadcast(action, player);
                    if (cfg.getDebug()) {
                        System.out.println("Broadcasted to "
                                + game.getName() + " except  "
                                + player.getLoginName() + ": "
                                + action);
                    }
                } else { // sendTo 
                    // just don't have time to do it properly
                    // TODO find good pullparsing implementaion for Java
                    // and use it everywhere we use JDOM now
                    
                    SAXBuilder sb = new SAXBuilder(false);
                    StringReader sr = new StringReader(action);
                    Document doc = sb.build(sr);
                    
                    Element root = doc.getRootElement();
                    
                    String recipient = 
                        root.getAttributeValue("recipient");
                    String toRoom =
                        root.getChild("invitation").getAttributeValue("toRoom");
                    
                    String msg = "<invitation toRoom=\"" + toRoom + "\"/>";                                
                        
                    game.sendMesageToPlayer(recipient, msg);
                    // gee! at last!
                    
                }
            }

            if (removedRoom) {
                removedRoom = false;
                actualRoom = null;
            }

            if (leftRoom) {
                leftRoom = false;
                actualRoom = null;
            }
            
            if (gameUnsetted) {
                gameUnsetted = false;
                game = null;
            }

        }        
    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            node which contains name of gamelet to load
     * @return XML messages to send (on per line)
     * @throws DbWrapperException
     */
    private String setGame(String id, Element name) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive
        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        // check dependecies and do the job
        if (!depsSetGame) {
            game = games.getGameByName(name.getText());
            if (game != null) {
                rooms = game.getRooms();
                depsSetGame = true;
                
                // pass chosen game to DbWrapper 
                // without that it doesn't do the job because knows 
                // nothing about to which collection should use 
                db.setGame(game.getName());
                
                return replyOK(id, "setGame");                

            } else {
                return replyFailed(id, "setGame",
                        "not such game on server, sorry!");

            }

        } else {
            // no such game exists
            return replyFailed(id, "setGame", "game already chosen");
        }

    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            node which contains login name passed in request
     * @param password
     *            node which contains passoword passed in request
     * @return XML messages to send (one per line)
     * @throws DbWrapperException
     */
    private String login(String id, Element name, Element password) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        if (password == null) {
            return replyFailed(id, "parseError", "password element missing");
        }
        
        // check dependecies and do the job
        if (depsSetGame && (!depsLoggedIn)) {            
            if (game.checkPlayer(name.getText())) {
                return replyFailed(id, "login", "you are already logged in");
            }

            if (!db.dbCheckLogin(name.getText(), password.getText())) {
                return replyFailed(id, "login", "incorrect login or password");
            }

            player.setLoginName(name.getText());

            depsLoggedIn = true;
            game.addPlayer(player);

            return replyOK(id, "login") + "\n"
                    + broadcastInGame("login", player.getLoginName());

        } else {
            return replyFailed(id, "login", "game not chosen");
        }
    }

    /**
     * Logout doesn't mean destroying connection. For example player can have
     * two account's and want to switch between them while the client is running
     * 
     * @param id
     *            Idicates for which request reply will be produced
     * @return XML messages to send (on per line)
     */
    private String logout(String id) {
        String xmlout = "";
        
        // check dependecies and do the job
        if (depsLoggedIn && depsSetGame) {
            game.delPlayer(player);
            rooms = null;
            depsSetGame = false;
            
            // when empty it means this is cleaning up call & don't have to reply
            if (!id.equals("")) {
                xmlout = xmlout + replyOK(id, "logout") + "\n"; 
            }

            // inform every one that you have gone 
            xmlout = broadcastInGame("logout", player.getLoginName()) + "\n"; 

            // depsInRoom 
            if (actualRoom != null) {
                player.leaveRoom();
                depsInRoom = false;
                //actualRoom = null;
                leftRoom = true;
                xmlout = xmlout + broadcastInRoom("leaveRoom", 
                        "<name>" + player.getLoginName()) + "</name>" + "\n"; 
            }

            // to be sure - not really needed
            db.unsetGame();
            gameUnsetted = true;

            depsLoggedIn = false;
            

            return xmlout;
            
        } else {
            return replyFailed(id, "logout", "not logged in");
        }
    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            node which contains login name passed in request
     * @param password
     *            node which contains password passed in request
     * @param passwordAgain
     *            node which contains passwordAgain in request
     * @param avatar
     *            node which contains avatar which is uuencoded image
     * @return XML messages to send (on per line)
     * @throws DbWrapperException
     */
    private String register(String id, Element name, Element password,
            Element passwordAgain, Element avatar) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        if (name.getText().equals("")) {
            return replyFailed(id, "parseError",
                    "name element mustn't be empty");
        }

        if (password == null) {
            return replyFailed(id, "parseError", "password element missing");
        }

        // hardcoded password minimum length
        if (password.getText().length() < 6) {
            return replyFailed(id, "parseError", "password too short");
        }

        if (passwordAgain == null) {
            return replyFailed(id, "parseError",
                    "passwordAgain element missing");
        }

        if (avatar == null) {
            return replyFailed(id, "parseError", "avatar element missing");
        }

        if (!password.getText().equals(passwordAgain.getText())) {
            return replyFailed(id, "register", "passwords do not match");

        }
        // check dependecies and do the job
        if (depsSetGame && (!depsLoggedIn)) {
            if (db.dbRegister(name.getText(), password.getText())) {
                player.setLoginName(name.getText());
                game.addPlayer(player);
                
                depsLoggedIn = true;

                return replyOK(id, "register") + "\n"
                        + broadcastInGame("login", player.getLoginName());
            } else {
                return replyFailed(id, "register", "user " + name.getText()
                        + " already exists");
            }

        } else {
            return replyFailed(id, "register", "game not chosen");
        }

    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @return XML messages to send (on per line)
     */
    private String getRooms(String id) {
        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn) {

            String xmlout = "<reply id=\"" + id + "\" type=\"getRooms\">";
            xmlout = xmlout + rooms.toString();

            xmlout = xmlout + "</reply>";

            return xmlout;
        } else {
            return replyFailed(id, "getRooms", "not logged in");

        }
    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            node element which contains room name passed in request
     * @return XML messages to send (on per line)
     */
    private String joinRoom(String id, Element name) {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        Room r = rooms.getRoomByName(name.getText());

        if (rooms.getRoomByName(name.getText()) == null) {
            return replyFailed(id, "joinRoom", "room " + name.getText()
                    + " doesn't exist");
        }
        
        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn && (!depsInRoom)) {
            
            if ( player.joinRoom(r) ) {
                actualRoom = player.getRoom();
                depsInRoom = true;                

                return replyOK(id, "joinRoom") + "\n"
                	+ broadcastInRoom("joinRoom", "<name>" + player.getLoginName() + "</name>");
            } else {
                return replyFailed(id, "joinRoom", "you must be invited to " +
                		"join this room");
            }

        } else { // TODO (minor) we should distinguish between reasons!
            if (depsInRoom) {
                return replyFailed(id, "joinRoom", "room already chosen"); 
            } else {
                return replyFailed(id, "joinRoom", "not logged in");
            }

        }
    }

    /**
     * Leaves actual room.
     * 
     * @param id
     *            Idicates for which request reply will be produced
     * @return XML messages to send (on per line)
     */
    private String leaveRoom(String id) {
        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn && depsInRoom) {

            depsInRoom = false;
            player.leaveRoom();
            // we use it because we want to set actualRoom to null
            // but still we need to broadcast some info to it!
            leftRoom = true;
            
            // TODO we should deal somehow if player was playing (was ByTable and/or 
            // game started
            

            return replyOK(id, "leaveRoom") + "\n"
                    + broadcastInRoom("leaveRoom", "<name>" + player.getLoginName() + "</name>");

        } else if ((!depsSetGame) || (!depsLoggedIn)) {
            return replyFailed(id, "leaveRoom", "not logged in");

        } else { // actualRoom not set
            return replyFailed(id, "leaveRoom", "no room chosen");

        }

    }

    
    /**
     * @param playerName
     * @param roomName
     * @return
     * @throws DbWrapperException
     */
    private String grantInvitation(String id, Element playerName, Element roomName) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (playerName == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        if (playerName.getText().equals("")) {
            return replyFailed(id, "parseError",
                    "name element mustn't be empty");
        }        
        
        if (roomName == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        if (roomName.getText().equals("")) {
            return replyFailed(id, "parseError",
                    "name element mustn't be empty");
        }        
                
        Room r = rooms.getRoomByName(roomName.getText());
        
        if ( r == null ) {
            return replyFailed(id, "grantInvite", "no such room");
        }
        
        // we can only grant invite to player which is actually logged in
        if ( !game.checkPlayer(playerName.getText())) {
            return replyFailed(id, "grantInvite", "player must be logged in" +
            		" order to grant him an invite");
        }
        
        if (depsSetGame && depsLoggedIn ) {      
            if (r.getOwner().equals(player.getLoginName())) {
                db.dbAddInvite(playerName.getText(), roomName.getText());
                r.grantInvite(playerName.getText());            
            
                return replyOK(id, "grantInvite") + "\n"  
                	+ sendInvitation(roomName.getText(), playerName.getText());
            } else {
                return replyFailed(id, "grantInvite", "you are not owner of this room");                
            }
            
        } else {
            return replyFailed(id, "grantInvite", "not logged in");
            
        }
    }
    
    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            Node which contains room name passed in request
     * @return XML messages to send (on per line)
     */
    private String getPlayersInRoom(String id, Element name) {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn) {
            Room r = rooms.getRoomByName(name.getText());

            if (r == null) {
                return replyFailed(id, "getPlayersInRoom",
                        "no such room exists");
            }

            String xmlout;
            xmlout = "<reply id=\"" + id + "\" type=\"getPlayersInRoom\">";
            xmlout = xmlout + r.getPlayersInRoom();
            xmlout = xmlout + "</reply>";

            return xmlout;

        } else {
            return replyFailed(id, "getPlayersInRoom", "not logged in");

        }
    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @return XML messages to send (on per line)
     */
    private String getPlayersInGame(String id) {
        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn) {
            String xmlout;
            xmlout = "<reply id=\"" + id + "\" type=\"getPlayersInGame\">";
            xmlout = xmlout + game.getPlayersInGame();
            xmlout = xmlout + "</reply>";

            return xmlout;

        } else {
            return replyFailed(id, "getPlayersInGame", "not logged in");

        }

    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            Node which contains room name to create passed in request
     * @param type
     *            Node which contains room type (public or private) passed in
     *            request
     * @return XML messages to send (on per line)
     * @throws DbWrapperException
     */
    private String newRoom(String id, Element name, Element type) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        if (type == null) {
            return replyFailed(id, "parseError", "type element missing");
        }

        if (!((type.getText().equals("public")) || (type.getText()
                .equals("private")))) {
            return replyFailed(id, "parseError",
                    "type can have only two values:" + "public or private");
        }

        // maybe room of given name already exists??
        // this is double checked. here and while adding to db 
        // but it is good to keep it in both places        
        if (rooms.getRoomByName(name.getText()) != null) {
            return replyFailed(id, "newRoom", "room name " + name.getText() 
                    + " already in use");           
            
        }

        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn && (!depsInRoom)) {
            if (db.dbAddRoom(name.getText(), player.getLoginName(), type
                    .getText())) {
                
                Room r = new Room(name.getText(), player.getLoginName(), 
                        type.getText(), null);
                
                rooms.addRoom(r);

                // we are first person in the room, doesn't need to
                // broadcastToRoom about joining to it!
                player.joinRoom(r);
                actualRoom = player.getRoom();
                depsInRoom = true;

                return replyOK(id, "newRoom") + "\n"
                        + broadcastInGame("newRoom", r.getName());
                
            } else {
                return replyFailed(id, "newRoom", "room name " + name.getText() 
                        + " already in use");
            }

        } else { //we should distinguish between reasons
            return replyFailed(id, "newRoom", "not logged in");

        }

    }

    /**
     * @param id
     *            Idicates for which request reply will be produced
     * @param name
     *            Node which contains room name to remove passed in request
     * @return XML messages to send (on per line)
     * @throws DbWrapperException
     */
    private String removeRoom(String id, Element name) throws DbWrapperException {
        // we are validating requests by hand because validating
        // by schema is quite expensive

        if (name == null) {
            return replyFailed(id, "parseError", "name element missing");
        }

        Room room = rooms.getRoomByName(name.getText());

        // maybe room of given name doesn't exist??
        if (room == null) {
            return replyFailed(id, "removeRoom", "room " + name.getText()
                    + " doesn't exist");
        }

        // maybe player is not owner of the room? (only owner of room
        // can delete it)

        if (!room.getOwner().equals(player.getLoginName())) {
            return replyFailed(id, "removeRoom",
                    "you are not the owner of room " + name.getText());
        }

        // check dependecies and do the job
        if (depsSetGame && depsLoggedIn) {
            Room r = rooms.getRoomByName(name.getText());

            // hmm not necessery
            String rrname = new String(r.getName());

            rooms.delRoom(r);
            r.removeMe();
            db.dbRemoveRoom(rrname);

            // the only reference in this thread to room in which
            // we were is actualRoom. we will set it to null
            // and let's GC takes care of the rest

            // removedRoom is used because we want to set actualRoom
            // to null, but we *still* need to broadcast sth to room before!
            depsInRoom = false;
            removedRoom = true;

            return replyOK(id, "removeRoom") + "\n"
                    + broadcastInGame("removeRoom", "<name>" + rrname + "</name>") + "\n"
                    + broadcastInRoom("removeRoom", "<name>" + rrname + "</name>");

        } else {
            return replyFailed(id, "removeRoom", "not logged in");

        }
    }

    /**
     * Get statistic - simple XPath query on NXD Not implemented yet
     * Should allow to pass XPath/XQuery queries?? or at least choose
     * range of "places"
     * 
     * 
     * @param id
     *            Idicates for which request reply will be produced *
     * @return XML messages to send (on per line)
     */
    private String getStats(String id) {

        return replyFailed(id, "getStats", "not implemented yet");

    }

    /**
     * Generates OK reply for given request type
     * 
     * @param id
     *            Idicates for which request reply will be produced
     * @param type
     *            Type of request for which reply will be generated
     * @return XML messages to send (on per line)
     */
    private String replyOK(String id, String type) {
        return "<reply id=\"" + id + "\" type=\"" + type + "\">" + "<ok/>"
                + "</reply>";
    }

    /**
     * Generates Failed reply for given request type, with given explaination
     * 
     * @param id
     *            Idicates for which request reply will be produced
     * @param type
     *            Type of request for which reply will be generated
     * @param reason
     *            Reason why failure occured
     * @return XML messages to send (on per line)
     */
    private String replyFailed(String id, String type, String reason) {
        return "<reply id=\"" + id + "\" type=\"" + type + "\">"
                + "<failed reason=\"" + reason + "\"/>" + "</reply>";
    }

    /**
     * Generates reply with given content
     * 
     * @param id
     * @param type
     * @param content
     * @return
     */
    private String reply(String id, String type, String content) {
        return "<reply id=\"" + id + "\" type=\"" + type + "\">"
        		+ content + "</reply>";
    }
    
    /**
     * Sends messgae to all but not "name" player in actual room
     * 
     * @param type
     *            Which type of request caused this broadcast
     * @param name
     *            What name node will contain
     * @return XML to broadcast
     */
    private String broadcastInRoom(String type, String content) {
        // yes i know i have hardcoded <name>
        return "<broadcastInRoom type=\"" + type + "\">" + content 
                + "</broadcastInRoom>";
    }

    /**
     * Send message to all but not "name" player in whole game
     * 
     * @param type
     *            Which type of request caused this broadcast
     * @param name
     *            What name node will contain
     * @return XML to broadcast
     */
    private String broadcastInGame(String type, String name) {
        // yes i know i have hardcoded <name>
        return "<broadcastInGame type=\"" + type + "\">" + "<name>" + name
                + "</name>" + "</broadcastInGame>";
    }

    /**
     * Special type of broadcastInRoom for chat purposes only
     * 
     * @param from
     *            Name of user who sends this message
     * @param msg
     *            Message to send
     * @return XML to broadcast
     */
    private String broadcastChatMsg(String from, String msg) {
        return "<broadcastInRoom type=\"chatMsg\">" + "<msg from=\"" + from
                + "\">" + msg + "</msg>" + "</broadcastInRoom>";
    }

    /**
     * 
     * @param roomName
     * @return
     */
    private String sendInvitation(String roomName, String to) {
        return "<sendTo recipient=\"" + to + "\">" +
        		"<invitation toRoom=\""+roomName+"\"/></sendTo>";        
    }
}

